#!/usr/bin/tclsh

# Version history: 	1.1 05.02.2002 LMA Added the configurable theme feature.
# 			1.2 03.04.2002 LMA Added the Site Map feature. (see the pagen.lib).
#
# Notes: 
# 	1. I used mkdir, a *nix command to create dirs. To make it win* compatible, this has to be changed. 
# 
# (c) 2002 Luis M. Arteaga <lmiguel@arteaga.com.mx>. This program is protected by the GPL license, either
#     version 2 or later. 
  
 
# Global Variable setting. 
source pagen.config

# Load database libraries
source db_lib.tcl

# Libraries
source pagen.lib

# Language selection
source $Clang_file

# Specific procedures 

proc Perror {Vproc_name Verror_number} {
 

 switch -exact -- $Verror_number {
          "101"	{ set Verr_text	"Directory not found." } 
       
 }
 puts "PaGEN Fatal Error: \[$Vproc_name - $Verror_number\] : $Verr_text"
 exit
}

proc Ph_search {Vaux_start Vlevel_start} {
global Vlabel
 # We start where we are now. 
 set Vreturn_value ""
 
 set Vaux_query "select * from pages where name='$Vaux_start'"
 #Pdebug Ph_search "Getting parent of $Vaux_start : $Vaux_query"

 array set Vnew [ Pdb_exec "$Vaux_query" ]   
 set Vlevel_up [ string repeat "../" $Vlevel_start ]
 if { [info exists Vnew(0,parent) ]  && ([string length $Vnew(0,parent)] > 0) } {
    set Vlink     "<a href=\"$Vlevel_up$Vnew(0,file).html\">$Vnew(0,title)</a>"
    # Complication: When both parent and children pages resides on the same directory
    # a hierarchy paradox occurs. Its like when a grown-up lives in the same house than
    # his parents. -lma
      array set Vaux_parent [ Pdb_exec "select category from pages where name = '$Vnew(0,parent)'"]
      if ![ string match $Vaux_parent(0,category) $Vnew(0,category)] {
        set Vnew_level [ expr $Vlevel_start +1 ]  
      } else {
        set Vnew_level $Vlevel_start
      }
    # End complicated code
    set Vreturn_value "[Ph_search $Vnew(0,parent) $Vnew_level] | $Vlink $Vreturn_value"
 } else {
    #Pdebug Ph_search  "no parent"
    set Vreturn_value "<a href=\"$Vlevel_up$Vaux_start.html\">$Vlabel(main_page)</a>"
 }
 return "$Vreturn_value"
}


proc Phtml_insert {Vout_handle Vsource_file} {
global Cbase_dir Vglobal_values
	
   set Vhtml_file "$Cbase_dir/../$Vsource_file"
   set Vhandle    [ open $Vhtml_file r ]
   
   foreach line [ split [ read $Vhandle ] \n ] {
      	# Dont forget the substitutions

   	foreach Vaux_subst [ array names Vglobal_values ] {
		# Avoid the ugly & problem
		set line [ string map [ list "@$Vaux_subst" "$Vglobal_values($Vaux_subst)"] $line]
	}
	# Special characters that trigger an action of the program
	if [ string match "*@site_map*" $line ] { Psite_map $Vout_handle $Vglobal_values(page_offset) }
		set line [string map [list "@site_map" ""] $line ]
        #
     	puts $Vout_handle $line
   }
   close $Vhandle
}

proc Ptoc_box {Vout_handle Vaux_start} {
global Vglobal_values Cmax_content Ctheme Vlabel

  # This procedure creates the "On this page" box.
 set Vglobal_values(box_title) "$Vlabel(this_page)"
 set Vglobal_values(box_content) "<br><small>"
 array set Vcontent_list [ Pdb_exec "select id,title from content where page='$Vaux_start' order by id asc" ]
 array set Vpage_details [ Pdb_exec "select file from pages where name='$Vaux_start'" ]
 set Vpage_counter 1
 set Vcounter 0
 if {$Vcontent_list(0,tups) > 0 } {

   for { set j 0 } { $j < $Vcontent_list(0,tups) } { incr j 1 } {
     if { $Vcounter >= $Cmax_content } {
	incr Vpage_counter 1
        set Vcounter 0
     }
     if { $Vpage_counter >1 } {
	set Vnew_file "$Vpage_details(0,file)$Vpage_counter.html"
     } else {
	set Vnew_file "$Vpage_details(0,file).html"
     }
     append Vglobal_values(box_content) "<li><a href=\"$Vnew_file#$Vcontent_list($j,id)\">$Vcontent_list($j,title)</a>"
     incr Vcounter 1
   }
 set Vglobal_values(icon) ""
 set Vglobal_values(box_id) ""
 Phtml_insert $Vout_handle "theme/$Ctheme/box.html"
 }


}

proc Poffset {Vdir} {
 # This procedure calculates the offset of a page from the origin point

 set Vnumber [ regsub -all "\/" "$Vdir" "a" result ]
 if [ string match "" $Vdir ] { 
        set mask 0
 } else {
	set mask 1
 }
 set Voffset [string repeat "../" [ expr $Vnumber + $mask ] ]
 Pdebug "Poffset" "Calculating offset of $Vdir: $Voffset"
 return $Voffset
}


proc Pnavi_box {Vout_handle Vaux_start Vaux_category} {
global Cbase_dir Vglobal_values Ctheme Vlabel

 # This procedure creates the navigation box.
 set Vglobal_values(box_title) "$Vlabel(nav)"

 # Get the related pages to this one. 
 set Vglobal_values(box_content) ""
 array set Vchildren [ Pdb_exec "select * from v_pages where parent = '$Vaux_start'" ]
 #Pdebug "Pnavi_box" "Found $Vchildren(0,tups) childrens of this page $Vaux_start, cat $Vaux_category"
 append Vglobal_values(box_content) "<small>\n"
 if {$Vchildren(0,tups) > 0 } {
    for { set j 0 } { $j < $Vchildren(0,tups) } { incr j 1 } {
        if [ string match $Vaux_category $Vchildren($j,category) ] {
                #Pdebug "Pnavi_box" "Same category, not changing dirs"
		append Vglobal_values(box_content) "-<a href=\""
        } else {
  	 	append Vglobal_values(box_content) "-<a href=\"$Vglobal_values(page_offset)"
		append Vglobal_values(box_content) "$Vchildren($j,directory)/"   
        }
 	append Vglobal_values(box_content) "$Vchildren($j,file).html\">"
	append Vglobal_values(box_content) "$Vchildren($j,title)</a><br>\n"
        #Pdebug "Pnavi_box" "Added children $Vchildren($j,file).html, category $Vchildren($j,category)"
    }
 set Vglobal_values(box_id) " "
 set Vglobal_values(icon) ""
 Phtml_insert $Vout_handle "theme/$Ctheme/box.html"
 }
}

proc Pnew_on_site {Vout_handle} {
global Vglobal_values Ctheme Vlabel

 # this procedure creates a box with the "New" things on this site
 # based on the last_modified time stamp on content table

 # This procedure should be run only once???
 array set Vnew [ Pdb_exec "select * from content order by last_modified desc limit 10" ]
 
 set    Vglobal_values(box_content) "<center><small>$Vlabel(newly_changed)<br></center>"

 # Prevent if there are no new pages on the system (may be a recently installed pagen?)
 set Vlimit 10
 set Vin_str ""
 if {$Vnew(0,tups) < 10 } {    set Vlimit $Vnew(0,tups) }
 if {$Vnew(0,tups) < 1  } {    set Vlimit -1}
 
 for { set j 0 } { $j < $Vlimit } { incr j 1} {
   lappend Vin_str $Vnew($j,page)
 }
 set Vin_str [ lsort -unique $Vin_str ]
 foreach Vaux_page "$Vin_str" {
	array set Vlink [ Pdb_exec "select directory,title,name,file from v_pages where name = '$Vaux_page'" ]
        append Vglobal_values(box_content) "<br><br><font class=\"MENUHEADER\">$Vlink(0,title)</font>\n"
	for { set j 0 } { $j < $Vlimit } { incr j 1} { 
	   if [ string match $Vaux_page $Vnew($j,page) ] {
            if [ string match "" $Vlink(0,directory) ] { set Vdelim "" } else  { set Vdelim "/" }
 
 	    append Vglobal_values(box_content) "<li><a class=\"MENU\" href=\"$Vglobal_values(page_offset)"
	    append Vglobal_values(box_content) "$Vlink(0,directory)"
	    append Vglobal_values(box_content) "$Vdelim"
            append Vglobal_values(box_content) "$Vlink(0,file)"
	    if { $Vnew($j,split_level) > 1 } {
                append Vglobal_values(box_content) "$Vnew($j,split_level)"
            }
	    append Vglobal_values(box_content) ".html#$Vnew($j,id)\">"
	    append Vglobal_values(box_content) "$Vnew($j,title)</a></li>\n"
	   }
	}

 }

 set Vglobal_values(box_title)   $Vlabel(new)
 set Vglobal_values(icon) 	 ""
 set Vglobal_values(box_id)      ""
 Phtml_insert $Vout_handle "theme/$Ctheme/box.html" 
}

proc Psplit_page_nav {Vout_handle Vstart Vlevel Vtotal_content Vlimit} {
global Cimage_url Vglobal_values
 # This procedure makes a horizontal navigation bar, depending on the number of 
 # pages that where to be splitted. 

 puts $Vout_handle "<table border=0 align=center bgcolor=#ffaabb cellspacing=0 width=70% cellpadding=2><tr><td>"
 puts $Vout_handle "<table bgcolor=#ffffff border=0 cellspacing=0 cellpadding=2 width=100% align=center><tr>"
 set Vtotal_pages [ expr ($Vtotal_content.0000)/ $Vlimit ]
 set Vtotal_pages [ expr int(ceil($Vtotal_pages)) ]
 if { $Vlevel > 1 } {
   # The following "if" has an explanation: I dont like that the first page is named "1". 
   if { $Vlevel == 2 } {
      set Vprev_page "$Vglobal_values(file).html"  
   } else {
      set Vprev_page "$Vglobal_values(file)[expr $Vlevel -1 ].html"
   }
   puts $Vout_handle "<td rowspan=2><a href=$Vprev_page><img border=0 src=$Cimage_url/back.jpg></a></td>"
 }
 puts $Vout_handle "<td colspan=$Vtotal_pages align=center>Por la cantidad de contenido, esta p&aacute;gina se ha "
 puts $Vout_handle "dividido en $Vtotal_pages partes.</td>"
 if { $Vlevel < $Vtotal_pages } {
   set Vnext_page "$Vglobal_values(file)[expr $Vlevel +1 ].html"
   puts $Vout_handle "<td rowspan=2><a href=$Vnext_page><img border=0 src=$Cimage_url/next.jpg></a></td>"
 }

 puts $Vout_handle "</tr>"

 for { set j 1 } { $j <= $Vtotal_pages } { incr j 1 } {
        if { $j > 1 } {
	  set Vaux_page "$Vglobal_values(file)$j.html"
        } else {
	  set Vaux_page "$Vglobal_values(file).html"
	}
        if { $Vlevel ==  $j } {
	   puts $Vout_handle "<td align=center bgcolor=\"#ffccdd\">$Vaux_page</td>" 	
	} else {
	   puts $Vout_handle "<td align=center><a href=$Vaux_page>$Vaux_page</a></td>"
	}
 }
 
 puts $Vout_handle "</tr></table>"
 puts $Vout_handle "</td></tr></table><br>"
}

proc Padd_child_content {Vparent_id} {

 # This procedure looks for children of this content. 
 
 set Vreturn "" 

 array set Vcount [ Pdb_exec "select id_child from extra_content where id='$Vparent_id'" ]
 if  { $Vcount(0,tups) > 0 } { 
    set    Vquery "select content from child_contents where id_child in"
    append Vquery " (select id_child from extra_content where id='$Vparent_id')" 
    array set Vchild_content [ Pdb_exec $Vquery ]
    for { set j 0 } { $j <  $Vcount(0,tups) } { incr j 1 } {
	append Vreturn $Vchild_content($j,content)
    }
 }

 return $Vreturn
}

proc Pcontent {Vout_handle Vstart} {
global Vglobal_values Cmax_content Cimage_url Ctheme

  array set Vcontents [ Pdb_exec "select * from content where page='$Vstart' order by priority,id asc" ]
  
  # Set the starting poing 
  set Vstart_point [ expr ($Vglobal_values(level)-1) * $Cmax_content ]

 
  # If the content is just too much, force to stay in this level and dinamically include
  # a new page. 
  if { [ expr $Vcontents(0,tups)-$Vstart_point] > $Cmax_content } {
    set Vaux_max [ expr $Cmax_content + $Vstart_point ]
    #Pdebug "Pcontent" "Too much content for a single page, forcing a split"
    Pforce_new_page $Vstart $Vglobal_values(level)
  } else {
    set Vaux_max $Vcontents(0,tups)
  }
  
  # Make a navigation bar, if there are more than one level.
   if { $Vcontents(0,tups) > $Cmax_content } {
     Psplit_page_nav $Vout_handle $Vstart $Vglobal_values(level) $Vcontents(0,tups) $Cmax_content
   }
 
  if { $Vcontents(0,tups) > 0 } {
     #Pdebug "Pcontent"  "Generating content of page $Vstart$Vglobal_values(level): Starting at $Vstart_point, until $Vaux_max"

     for { set j $Vstart_point } { $j < $Vaux_max } { incr j 1 } {
       set Vglobal_values(box_title)   	$Vcontents($j,title)
       set Vglobal_values(box_content) 	$Vcontents($j,content)
       ## Add additional content , if any. 
       append Vglobal_values(box_content) [ Padd_child_content $Vcontents($j,id) ]
       set Vglobal_values(box_id)      	$Vcontents($j,id)
       set Vglobal_values(date) 	$Vcontents($j,date)
       # create the icon link 
 	if ![ info exists Vcontents($j,icon) ] {
          set Vglobal_values(icon) ""
	} else {
	  if {[ string match "none" $Vcontents($j,icon) ] || [ string match "" $Vcontents($j,icon) ]} {
		# none is a valid way to say there is no icon
	        set Vglobal_values(icon) "" 
          } else  { 
                #Pdebug "Pcontent" "Setting icon of this content to $Cimage_url/$Vcontents($j,icon)" 
         	set Vglobal_values(icon) "<img src=$Cimage_url/$Vcontents($j,icon) border=0 align=right>"
	  }
	}

       if { [ string length $Vcontents($j,author) ] > 0 } {
         set Vglobal_values(author) "Esta es una colaboracion de : &nbsp;$Vcontents($j,author)"
       } else {
	 set Vglobal_values(author) "&nbsp;" 
       }
       Phtml_insert $Vout_handle "theme/$Ctheme/box.html"
       puts $Vout_handle "<br>"

       # If the actual level is different, then update to new page.
       if { $Vcontents($j,split_level) !=  $Vglobal_values(level) } {
           Pdebug "Pcontent" "Updating this content to be on a different level. New level: $Vglobal_values(level)"
           Pdb_exec "update content set split_level='$Vglobal_values(level)' where id='$Vcontents($j,id)'"
       }
     }
  } else {
      #Pdebug "Pcontent" "No content found for page $Vstart"
  }
   # Make a navigation bar, if there are more than one level.
   if { $Vcontents(0,tups) > $Cmax_content } {
     Psplit_page_nav $Vout_handle $Vstart $Vglobal_values(level) $Vcontents(0,tups) $Cmax_content
   }
}

proc Pforce_new_page {Vprev_short Vprev_level} {
global Vpages

 # There MUST be a more elegant way to include pages dinamically, but so far
 # this is the easiest I can think about right now. I am tired. -lma
 # Why include pages at run time? If the content is too big for a page, then
 # I would like to split it into two (or more) pages automatically. Thats why. 

 array set Vold_page [ Pdb_exec "select * from pages where name='$Vprev_short'" ]

 set  Vpages($Vpages(0,tups),icon) $Vold_page(0,icon)
 Pdebug "Pforce_new_page" "Setting new splitted page name to $Vold_page(0,name) ($Vpages(0,tups))"
 set  Vpages($Vpages(0,tups),name) $Vold_page(0,name)
 set  Vpages($Vpages(0,tups),category) $Vold_page(0,category)
 set  Vpages($Vpages(0,tups),level) [ expr $Vprev_level + 1]
 set  Vpages($Vpages(0,tups),parent) $Vold_page(0,parent)
 set  Vpages($Vpages(0,tups),file)   $Vold_page(0,file)
 set  Vpages($Vpages(0,tups),short)  "$Vprev_short"
 set  Vpages($Vpages(0,tups),title)  "$Vold_page(0,title) \| $Vpages($Vpages(0,tups),level)a parte" 
 incr Vpages(0,tups) 1 

}

proc Pupdate_last_modified {Vpage} {

 # This procedure updates the last time this page was modified
 set Vtime_now [ clock format [ clock seconds ] -format "%Y%m%d%H%M%S" ]
 set Vquery "update pages set last_modified='$Vtime_now' where name='$Vpage'"
 Pdb_exec $Vquery
 Pdebug "Pupdate_last_modified" "Setting last modification time for page $Vpage to $Vtime_now"
}
 
proc Pcheck_which_pages {} {
global argv argc

  # This procedure checks if there was a page specification. if not, return a null value
  # to the whereclause
  if  { $argc > 0 } {
      switch -exact -- [ lindex $argv 0 ] {
      		"-allnew"  { 
                            Pdebug "Pcheck_which_pages" "Setting generate only for new pages"
			    return " where last_modified > (select last_run from control)"				
		           }
		"-page"	   { return " where name = '[lindex $argv 1]' " }
 		"-allsite" {
                                return " where (1 = 1) "
                           }
      }
  } else {
  	return ""
  }

}


proc Pblock_modules {Vpage_handle} {
global Cmodules_dir Vlabel Ctheme Vglobal_values

 # This procedure includes the blocks found on the modules directory
 
 if ![file isdirectory $Cmodules_dir] { Perror "Pblock_modules" 101 }

 puts $Vpage_handle "<br>"

 # Read the files corresponding to block modules
 foreach Vblock_file [ glob -nocomplain -types f -directory "$Cmodules_dir" "*blk" ] {
 
  # Open the files and write them directly on the output - without changes. 
  Pdebug "Pblock_modules" "Adding the block module [file tail $Vblock_file]" 
  set Vtemp_fl_handle [ open "$Vblock_file" "r" ]
  set Vglobal_values(box_title)   "$Vlabel(module)"
  set Vglobal_values(icon)        ""
  set Vglobal_values(box_id)      "M"
  set    Vglobal_values(box_content) "<center>[read $Vtemp_fl_handle]</center>"   
  Phtml_insert $Vpage_handle "theme/$Ctheme/box.html"
  close $Vtemp_fl_handle 
 }
 
 return;
}

proc Ppage_gen {} {
global Cout_dir Vglobal_values Cimage_url Vpages Ctheme

  # This procedure generates the pages! 
  # Warning! All pages will be overwritten. Do not edit manually!!!"
  #Pdebug "Ppage_gen" "Generating pages..."
  array set Vpages [ Pdb_exec "select * from pages [Pcheck_which_pages] and render != 'no' " ]
  if { $Vpages(0,tups) > 0 } {
  	for { set i 0 } { $i < $Vpages(0,tups) } { incr i 1 } { 

		# real creation
                # Generate auto-level for split, if does not exist. 
                if ![ info exists Vpages($i,level) ] {
			set Vpages($i,level) 1
		}

		# Step 1. Test if the dir exists. 
		  array set Vcat [ Pdb_exec "select * from categories where category = '$Vpages($i,category)'" ]                

          	Pdebug "Ppage_gen" "Generating page $Vpages($i,name) ($Vcat(0,directory)/$Vpages($i,file).html)"

		  set Vdir_2b_tested "$Cout_dir/$Vcat(0,directory)"
		  Pdir_test $Vdir_2b_tested;
     
                # Step 2. Create the page
                  if { $Vpages($i,level) > 1 } {
			  set    Vpage_file "$Vdir_2b_tested/$Vpages($i,file)$Vpages($i,level)"
		  } else {
			  set    Vpage_file "$Vdir_2b_tested/$Vpages($i,file)"
		  }
                  append Vpage_file ".html"
		  set    Vpage_handle [ open $Vpage_file w ]
                  puts   $Vpage_handle "<!-- This page was automatically generated by PaGEN v1.2a -->"
                  

 		# Step 3. Generate values
    
   		set Vglobal_values(page_title) 	$Vpages($i,title)
   		set Vglobal_values(page_name)   $Vpages($i,name)
		set Vglobal_values(time_now)	[ clock format [ clock seconds ] -format "%d.%m.%Y %H:%M" ]
		set Vglobal_values(hierarchy)   [ Ph_search "$Vpages($i,name)" 0]
		set Vglobal_values(image_url)   $Cimage_url
		set Vglobal_values(author) 	"&nbsp;"
		set Vglobal_values(date) 	""
                set Vglobal_values(level) 	$Vpages($i,level)
		set Vglobal_values(file)        $Vpages($i,file)
		set Vglobal_values(page_offset)  [ Poffset $Vcat(0,directory) ]

                # Step 4. Insert the top HTML
		  Phtml_insert $Vpage_handle "theme/$Ctheme/top.html"

		# Step 5. Create the middle. 
		# Until now, the main page layout is totally hardcoded, but I would like
		# to implement an easy way to modify it. 
		
		puts $Vpage_handle "<table border=0 cellspacing=0 cellpadding=2 width=100%><tr>"
		puts $Vpage_handle "<td valign=top width=20%>"
                Pnavi_box $Vpage_handle $Vpages($i,name) $Vpages($i,category)
                puts $Vpage_handle "<br>"
                Ptoc_box $Vpage_handle $Vpages($i,name)
                puts $Vpage_handle "<br>"
		Pnew_on_site $Vpage_handle 

                # Include the block modules 
		Pblock_modules $Vpage_handle

		puts $Vpage_handle "</td><td valign=top width=80%>"

		  ## content of the page
    		  Pcontent $Vpage_handle $Vpages($i,name)

		puts $Vpage_handle "</td>"
  		puts $Vpage_handle "</tr></table>"


		# Step one but last: Insert the bottom
		  Phtml_insert $Vpage_handle "theme/$Ctheme/bot.html"

		# Last step. Close the page
		  close $Vpage_handle
	          Pupdate_last_modified $Vpages($i,name)
		  exec chmod 777  $Vpage_file		
        }
  }


  Pdebug "Ppage_gen" "End page generation."
}

proc Pcontrol {} {
  # This procedure updates the last known time pagen was run

  set Vtime_now [  clock format [ clock seconds ] -format "%Y%m%d%H%M%S" ] 
  Pdb_exec "update control set last_run='$Vtime_now'"
  return 0
}

proc main {} {
  global Cout_dir Vglobal_values
  set Vglobal_values(version) "1.2a"
  Pdb_connect
  Pdebug main "Generating pages into $Cout_dir"
  Pdir_test $Cout_dir
  Ppage_gen
  Pcontrol
  Pdb_disconnect
  Pdb_stats
}

# Run!
main
